import { configure, getLogger, Logger, addLayout } from 'log4js';
import * as moment from 'moment';
import { injectable } from 'inversify';

export const Slogger = Symbol('Slogger');

export enum IpcMessageType {
    Request = "Request",
    Response = "Response",
    Notification = "Notification",
    Error = "Error"
}

export interface Slogger {

    info(message: string, ...args: any[]): void;

    error(message: string, ...args: any[]): void;

    logIpcCall(id: number, type: IpcMessageType, name: string, params: any): void;

    logState(state: any) : void;
}

@injectable()
export class SloggerDefault implements Slogger {

    logState(state: any): void {
        throw new Error("Method not implemented.");
    }

    logIpcCall(id: number, type: IpcMessageType, name: string, params: any): void {
        throw new Error("Method not implemented.");
    }

    info(message: string, ...args: any[]): void {
        console.info(message, args);
    }

    error(message: string, ...args: any[]): void {
        console.error(message, args);
    }

}

@injectable()
export class SloggerNode implements Slogger {
    
    readonly logger: Logger;

    readonly APPLICATION: string = "slogger-ui";

    constructor() {

        addLayout('json', (config) => {
            return (logEvent) => { 
                return JSON.stringify( {                     
                    ts: (logEvent.startTime).toISOString().slice(0, -1),
                    app: this.APPLICATION,
                    context: logEvent.context?.type ?? "plain",
                    component: logEvent.categoryName,
                    loglevel: logEvent.level.levelStr,
                    payload: (
                        (ctx) => { if(ctx === null || ctx === undefined) 
                                    return { message: logEvent.data[0] }
                                else
                                    return JSON.parse(logEvent.data[0])
                        })(logEvent.context.type) 
                });
            }
          });

        configure({

            appenders: { 
                file: { type: 'file', 
                        filename: `${this.APPLICATION}_${moment().format('YYYY-MM-DD_hh-mm')}.slog`,
                        layout: { type: 'json', separator: ',' } }, 
                out: { type: 'stdout' }
            }, 
            categories: { default: { appenders: ['file', 'out'], level: 'debug' } }
        });

        this.logger = getLogger();
    }

    logIpcCall(id: number, type: IpcMessageType, name: string, params: any): void {

        this.logger.addContext('type', 'ipc');
        if(type === IpcMessageType.Error)
        {
            this.error( JSON.stringify({ callid: id, msg_type: type, msg_name: name, params: params }));
        }
        else
        {
            this.info( JSON.stringify({ callid: id, msg_type: type, msg_name: name, params: params }));
        }
        this.logger.removeContext('type');

    }

    logState(state: any): void {
        this.logger.addContext('type', 'state');
        this.info(JSON.stringify( { message: state} ));
        this.logger.removeContext('type');
    }

    info(message: string, ...args: any[]): void {
        this.logger.info(message);
    }

    error(message: string, ...args: any[]): void {
        this.logger.error(message);
    }

}